@@ -22,29 +22,33 @@
  */
 
 #include "config.h"
-#include <sys/types.h>
-#include <dirent.h>
 #include <string.h>
 #include <stdlib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-#include <glade/glade.h>
+#include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include "nautilus-sendto-plugin.h"
 
 #define NAUTILUS_SENDTO_GCONF		"/desktop/gnome/nautilus-sendto"
 #define NAUTILUS_SENDTO_LAST_MEDIUM	NAUTILUS_SENDTO_GCONF"/last_medium"
 #define NAUTILUS_SENDTO_LAST_COMPRESS	NAUTILUS_SENDTO_GCONF"/last_compress"
-#define NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT 10000
+#define NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT_SECONDS 10
 
-#define UNINSTALLED_PLUGINDIR "plugins/.libs"
+#define UNINSTALLED_PLUGINDIR "plugins/removable-devices"
+#define UNINSTALLED_SOURCE "nautilus-sendto-command.c"
+
+enum {
+	COLUMN_ICON,
+	COLUMN_DESCRIPTION,
+	NUM_COLUMNS,
+};
 
 /* Options */
-static gchar *default_url = NULL;
 static char **filenames = NULL;
 
-gboolean force_user_to_compress = FALSE;
 GList *file_list = NULL;
+gboolean has_dirs = FALSE;
 GList *plugin_list = NULL;
 GHashTable *hash ;
 guint option = 0;
@@ -72,7 +76,6 @@ struct _NS_ui {
 };
 
 static const GOptionEntry entries[] = {
-	{ "default-dir", 'd', 0, G_OPTION_ARG_FILENAME, &default_url, N_("Default folder to use"), NULL },
 	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, "Movies to index", NULL },
 	{ NULL }
 };
@@ -257,10 +260,9 @@ status_label_clear (gpointer data)
 }
 
 static void
-send_button_cb (GtkWidget *widget, gpointer data)
+send_button_cb (GtkWidget *widget, NS_ui *ui)
 {
-	NS_ui *ui = (NS_ui *) data;
-	gchar *f, *error;
+	char *f, *error;
 	NstPlugin *p;
 	GtkWidget *w;
 
@@ -286,9 +288,9 @@ send_button_cb (GtkWidget *widget, gpointer data)
 			g_free (error);
 			gtk_label_set_markup (GTK_LABEL (ui->status_label), message);
 			g_free (message);
-			ui->status_timeoutid = g_timeout_add (NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT,
-							      status_label_clear,
-							      ui);
+			ui->status_timeoutid = g_timeout_add_seconds (NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT_SECONDS,
+								      status_label_clear,
+								      ui);
 			gtk_widget_show (ui->status_image);
 			gtk_widget_show (ui->status_box);
 			gtk_widget_set_sensitive (ui->dialog, TRUE);
@@ -299,7 +301,7 @@ send_button_cb (GtkWidget *widget, gpointer data)
 	gconf_client_set_string (gconf_client, 
 				NAUTILUS_SENDTO_LAST_MEDIUM, p->info->id, NULL);
 
-	if (force_user_to_compress){
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ui->pack_checkbutton))){
 		f = pack_files (ui);
 		if (f != NULL) {
 			GList *packed_file = NULL;		
@@ -313,48 +315,30 @@ send_button_cb (GtkWidget *widget, gpointer data)
 			gtk_widget_set_sensitive (ui->dialog, TRUE);
 			return;
 		}
-	}else{
-		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ui->pack_checkbutton))){
-			f = pack_files (ui);
-			if (f != NULL) {
-				GList *packed_file = NULL;
-				packed_file = g_list_append (packed_file, f);
-				if (!p->info->send_files (p, w, packed_file)) {
-					g_list_free (packed_file);
-					return;
-				}
-				g_list_free (packed_file);
-			} else {
-				gtk_widget_set_sensitive (ui->dialog, TRUE);
-				return;
-			}
-		} else {
-			if (!p->info->send_files (p, w, file_list)) {
-				g_list_foreach (file_list, (GFunc) g_free, NULL);
-				g_list_free (file_list);
-				file_list = NULL;
-				return;
-			}
+	} else {
+		if (!p->info->send_files (p, w, file_list)) {
+			g_list_foreach (file_list, (GFunc) g_free, NULL);
 			g_list_free (file_list);
 			file_list = NULL;
+			return;
 		}
+		g_list_free (file_list);
+		file_list = NULL;
 	}
 	destroy_dialog (NULL,NULL);
 }
 
 static void
-send_if_no_pack_cb (GtkWidget *widget, gpointer data)
+send_if_no_pack_cb (GtkWidget *widget, NS_ui *ui)
 {
-	NS_ui *ui = (NS_ui *) data;
-
-	if (force_user_to_compress || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton))) {
-		if (force_user_to_compress) {
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton))) {
+		if (GTK_WIDGET_IS_SENSITIVE (ui->pack_entry)) {
 			gtk_widget_grab_focus (ui->pack_entry);
 		} else {
 			gtk_widget_grab_focus (ui->pack_checkbutton);
 		}
 	} else {
-		send_button_cb (widget, data);
+		send_button_cb (widget, ui);
 	}
 }
 
@@ -382,9 +366,11 @@ toggle_pack_check (GtkWidget *widget, NS_ui *ui)
 }
 
 static void
-option_changed (GtkComboBox *cb, gpointer data){
-	NS_ui *ui = (NS_ui *) data ;
-	GList *aux;	
+option_changed (GtkComboBox *cb, NS_ui *ui)
+{
+	GList *aux;
+	NstPlugin *p;
+	gboolean supports_dirs = FALSE;
 
 	aux = g_list_nth (ui->contact_widgets, option);
 	option = gtk_combo_box_get_active (GTK_COMBO_BOX(cb));
@@ -393,10 +379,26 @@ option_changed (GtkComboBox *cb, gpointer data){
 	gtk_widget_show ((GtkWidget *) aux->data);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (ui->send_to_label), aux->data);
+
+	p = (NstPlugin *) g_list_nth_data (plugin_list, option);
+	supports_dirs = (p->info->capabilities & NAUTILUS_CAPS_SEND_DIRECTORIES);
+
+	if (has_dirs == FALSE || supports_dirs != FALSE) {
+		gboolean toggle;
+
+		toggle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton));
+		gtk_widget_set_sensitive (ui->pack_combobox, toggle);
+		gtk_widget_set_sensitive (ui->pack_entry, toggle);
+		gtk_widget_set_sensitive (ui->pack_checkbutton, TRUE);
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton), TRUE);
+		gtk_widget_set_sensitive (ui->pack_checkbutton, FALSE);
+	}
 }
 
 static void
-set_contact_widgets (NS_ui *ui){
+set_contact_widgets (NS_ui *ui)
+{
 	GList *aux ;
 	GtkWidget *w;
 	NstPlugin *p;
@@ -406,46 +408,50 @@ set_contact_widgets (NS_ui *ui){
 	for (aux = plugin_list; aux; aux = aux->next){
 		p = (NstPlugin *) aux->data;
 		w = p->info->get_contacts_widget(p);
-		gtk_box_pack_end_defaults (GTK_BOX(ui->hbox_contacts_ws),w);
+		gtk_box_pack_end (GTK_BOX(ui->hbox_contacts_ws),w, TRUE, TRUE, 0);
 		gtk_widget_hide (GTK_WIDGET(w));
 		ui->contact_widgets = g_list_append (ui->contact_widgets, w);
 		if (GTK_IS_ENTRY (w)) {
-			g_signal_connect (G_OBJECT (w), "activate",
-					G_CALLBACK (send_if_no_pack_cb), ui);
+			g_signal_connect_after (G_OBJECT (w), "activate",
+						G_CALLBACK (send_if_no_pack_cb), ui);
 		}
 	}
 }
 
-static void
-set_model_for_options_combobox (NS_ui *ui){
+static gboolean
+set_model_for_options_combobox (NS_ui *ui)
+{
 	GdkPixbuf *pixbuf;
         GtkTreeIter iter;
-        GtkTreeStore *model;
+        GtkListStore *model;
 	GtkIconTheme *it;
 	GtkCellRenderer *renderer;
 	GList *aux;
 	NstPlugin *p;
-	gchar *last_used = NULL;
+	char *last_used = NULL;
 	int i = 0;
+	gboolean last_used_support_dirs = FALSE;
 
 	it = gtk_icon_theme_get_default ();
 
-	model = gtk_tree_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
+	model = gtk_list_store_new (NUM_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING);
 
 	last_used = gconf_client_get_string (gconf_client,
 			NAUTILUS_SENDTO_LAST_MEDIUM, NULL);
 
-	for (aux = plugin_list; aux; aux = aux->next){
+	for (aux = plugin_list; aux; aux = aux->next) {
 		p = (NstPlugin *) aux->data;
 		pixbuf = gtk_icon_theme_load_icon (it, p->info->icon, 16, 
 						   GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
-		gtk_tree_store_append (model, &iter, NULL);
-		gtk_tree_store_set (model, &iter,
-					0, pixbuf,
-					1, p->info->description,
+		gtk_list_store_append (model, &iter);
+		gtk_list_store_set (model, &iter,
+					COLUMN_ICON, pixbuf,
+					COLUMN_DESCRIPTION, _(p->info->description),
 					-1);
-		if (last_used != NULL && !strcmp(last_used, p->info->id))
+		if (last_used != NULL && !strcmp(last_used, p->info->id)) {
 			option = i;
+			last_used_support_dirs = (p->info->capabilities & NAUTILUS_CAPS_SEND_DIRECTORIES);
+		}
 		i++;
 	}
 	g_free(last_used);
@@ -458,27 +464,28 @@ set_model_for_options_combobox (NS_ui *ui){
                                     FALSE);
         gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (ui->options_combobox), 
 					renderer,
-                                        "pixbuf", 0,
+                                        "pixbuf", COLUMN_ICON,
                                         NULL);		
         renderer = gtk_cell_renderer_text_new ();
+        g_object_set (G_OBJECT (renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->options_combobox),
                                     renderer,
                                     TRUE);
         gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (ui->options_combobox), 
 					renderer,
-                                        "text", 1,
+                                        "text", COLUMN_DESCRIPTION,
                                         NULL);
 
 	g_signal_connect (G_OBJECT (ui->options_combobox), "changed",
 			  G_CALLBACK (option_changed), ui);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (ui->options_combobox), option);
+
+	return last_used_support_dirs;
 }
 
 static void
-pack_entry_changed_cb (GObject *object,
-		       GParamSpec *spec,
-		       NS_ui *ui)
+pack_entry_changed_cb (GObject *object, GParamSpec *spec, NS_ui *ui)
 {
 	gboolean send_enabled;
 
@@ -498,32 +505,36 @@ pack_entry_changed_cb (GObject *object,
 static void
 nautilus_sendto_create_ui (void)
 {
-	GladeXML *app;	
-	gint toggle;
+	GtkBuilder *app;	
+	GError* error = NULL;
 	NS_ui *ui;
 	gboolean one_file = FALSE;
+	gboolean supports_dirs;
 
-	app = glade_xml_new (GLADEDIR "/" "nautilus-sendto.glade", NULL, NULL);
+	app = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (app, UIDIR "/" "nautilus-sendto.ui", &error))	{
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+	}
 
 	ui = g_new0 (NS_ui, 1);
 
-	ui->hbox_contacts_ws = glade_xml_get_widget (app, "hbox_contacts_widgets");
-	ui->send_to_label = glade_xml_get_widget (app, "send_to_label");
-	ui->options_combobox = glade_xml_get_widget (app, "options_combobox");
-	ui->dialog = glade_xml_get_widget (app, "nautilus_sendto_dialog");
-	ui->cancel_button = glade_xml_get_widget (app, "cancel_button");
-	ui->send_button = glade_xml_get_widget (app, "send_button");
-	ui->pack_combobox = glade_xml_get_widget (app, "pack_combobox");	
-	ui->pack_entry = glade_xml_get_widget (app, "pack_entry");
-	ui->pack_checkbutton = glade_xml_get_widget (app, "pack_checkbutton");
-	ui->status_box = glade_xml_get_widget (app, "status_box");
-	ui->status_label = glade_xml_get_widget (app, "status_label");
-	ui->status_image = glade_xml_get_widget (app, "status_image");
+	ui->hbox_contacts_ws = GTK_WIDGET (gtk_builder_get_object (app, "hbox_contacts_widgets"));
+	ui->send_to_label = GTK_WIDGET (gtk_builder_get_object (app, "send_to_label"));
+	ui->options_combobox = GTK_WIDGET (gtk_builder_get_object (app, "options_combobox"));
+	ui->dialog = GTK_WIDGET (gtk_builder_get_object (app, "nautilus_sendto_dialog"));
+	ui->cancel_button = GTK_WIDGET (gtk_builder_get_object (app, "cancel_button"));
+	ui->send_button = GTK_WIDGET (gtk_builder_get_object (app, "send_button"));
+	ui->pack_combobox = GTK_WIDGET (gtk_builder_get_object (app, "pack_combobox"));	
+	ui->pack_entry = GTK_WIDGET (gtk_builder_get_object (app, "pack_entry"));
+	ui->pack_checkbutton = GTK_WIDGET (gtk_builder_get_object (app, "pack_checkbutton"));
+	ui->status_box = GTK_WIDGET (gtk_builder_get_object (app, "status_box"));
+	ui->status_label = GTK_WIDGET (gtk_builder_get_object (app, "status_label"));
+	ui->status_image = GTK_WIDGET (gtk_builder_get_object (app, "status_image"));
 
  	gtk_combo_box_set_active (GTK_COMBO_BOX(ui->pack_combobox), 
  		gconf_client_get_int(gconf_client, 
  				NAUTILUS_SENDTO_LAST_COMPRESS, NULL));
- 
 
 	if (file_list != NULL && file_list->next != NULL)
 		one_file = FALSE;
@@ -535,7 +546,7 @@ nautilus_sendto_create_ui (void)
 	if (one_file) {
 		char *filepath = NULL, *filename = NULL;
 
-		filepath = g_filename_from_uri ((gchar *)file_list->data,
+		filepath = g_filename_from_uri ((char *)file_list->data,
 				NULL, NULL);
 
 		if (filepath != NULL)
@@ -555,7 +566,7 @@ nautilus_sendto_create_ui (void)
 	}
 
 	set_contact_widgets (ui);
-	set_model_for_options_combobox (ui);
+	supports_dirs = set_model_for_options_combobox (ui);
 	g_signal_connect (G_OBJECT (ui->dialog), "destroy",
                           G_CALLBACK (destroy_dialog), NULL);
 	g_signal_connect (G_OBJECT (ui->cancel_button), "clicked",
@@ -566,13 +577,15 @@ nautilus_sendto_create_ui (void)
 			  G_CALLBACK (send_button_cb), ui);
 	g_signal_connect (G_OBJECT (ui->pack_entry), "notify::text",
 			  G_CALLBACK (pack_entry_changed_cb), ui);
+	g_signal_connect (G_OBJECT (ui->pack_checkbutton), "toggled",
+			  G_CALLBACK (toggle_pack_check), ui);
+
+	if (has_dirs == FALSE || supports_dirs != FALSE) {
+		gboolean toggle;
 
-	if (force_user_to_compress == FALSE){
 		toggle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton));
 		gtk_widget_set_sensitive (ui->pack_combobox, toggle);
 		gtk_widget_set_sensitive (ui->pack_entry, toggle);
-		g_signal_connect (G_OBJECT (ui->pack_checkbutton), "toggled",
-				  G_CALLBACK (toggle_pack_check), ui);
 	} else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton), TRUE);
 		gtk_widget_set_sensitive (ui->pack_checkbutton, FALSE);
@@ -582,20 +595,14 @@ nautilus_sendto_create_ui (void)
 
 }
 
-static gboolean
-nautilus_sendto_plugin_init (void)
+static void
+nautilus_sendto_plugin_dir_process (const char *plugindir)
 {
 	GDir *dir;
 	const char *item;
 	NstPlugin *p = NULL;
 	gboolean (*nst_init_plugin)(NstPlugin *p);
 	GError *err = NULL;
-	const char *plugindir;
-
-	if (g_file_test (UNINSTALLED_PLUGINDIR, G_FILE_TEST_IS_DIR) != FALSE)
-		plugindir = UNINSTALLED_PLUGINDIR;
-	else
-		plugindir = PLUGINDIR;
 
 	dir = g_dir_open (plugindir, 0, &err);
 
@@ -603,7 +610,6 @@ nautilus_sendto_plugin_init (void)
 		g_warning ("Can't open the plugins dir: %s", err ? err->message : "No reason");
 		if (err)
 			g_error_free (err);
-		return FALSE;
 	} else {
 		while ((item = g_dir_read_name(dir))) {
 			if (g_str_has_suffix (item, SOEXT)) {
@@ -637,6 +643,38 @@ nautilus_sendto_plugin_init (void)
 		}
 		g_dir_close (dir);
 	}
+}
+
+static gboolean
+nautilus_sendto_plugin_init (void)
+{
+	if (g_file_test (UNINSTALLED_PLUGINDIR, G_FILE_TEST_IS_DIR) != FALSE) {
+		/* Try to load the local plugins */
+		GError *err = NULL;
+		GDir *dir;
+		const char *item;
+
+		dir = g_dir_open ("plugins/", 0, &err);
+		if (dir == NULL) {
+			g_warning ("Can't open the plugins dir: %s", err ? err->message : "No reason");
+			if (err)
+				g_error_free (err);
+			return FALSE;
+		}
+		while ((item = g_dir_read_name(dir))) {
+			char *plugindir;
+
+			plugindir = g_strdup_printf ("plugins/%s/.libs/", item);
+			if (g_file_test (plugindir, G_FILE_TEST_IS_DIR) != FALSE)
+				nautilus_sendto_plugin_dir_process (plugindir);
+			g_free (plugindir);
+		}
+		g_dir_close (dir);
+	}
+
+	if (g_list_length (plugin_list) == 0)
+		nautilus_sendto_plugin_dir_process (PLUGINDIR);
+
 	return g_list_length (plugin_list) != 0;
 }
 
@@ -682,56 +720,28 @@ nautilus_sendto_init (void)
 	if (g_module_supported() == FALSE)
 		g_error ("Could not initialize gmodule support");
 
-	if (default_url == NULL) {
-		default_url = g_get_current_dir ();
-	}
-
 	for (i = 0; filenames != NULL && filenames[i] != NULL; i++) {
-		const char *filename;
- 		char *path;
+		GFile *file;
+		char *filename, *escaped, *uri;
 
-		filename = filenames[i];
- 
- 		if (g_str_has_prefix (filename, "file://")) {
- 			char *escaped;
+		file = g_file_new_for_commandline_arg (filenames[i]);
+		filename = g_file_get_path (file);
+		if (filename == NULL)
+			continue;
 
-			/* Make sure URIs don't contain ampersands */
- 			escaped = escape_ampersands (filename);
- 			if (escaped == NULL)
-				file_list = g_list_prepend (file_list,
-							    g_strdup (filename));
-			else
-				file_list = g_list_prepend (file_list, escaped);
- 			path = g_filename_from_uri (filename, NULL, NULL);
- 			if (path != NULL
- 			    && g_file_test (path, G_FILE_TEST_IS_DIR)) {
- 				force_user_to_compress = TRUE;
- 			}
- 			g_free (path);
- 		} else {
- 			char *uri, *escaped;
+		if (g_file_test (filename, G_FILE_TEST_IS_DIR) != FALSE)
+			has_dirs = TRUE;
 
- 			if (filename[0] != G_DIR_SEPARATOR) {
- 				path = g_build_filename (default_url,
-						filename, NULL);
-			} else {
-				path = g_strdup (filename);
-			}
- 
- 			uri = g_filename_to_uri (path, NULL, NULL);
-			escaped = escape_ampersands (uri);
-			if (escaped) {
-				file_list = g_list_prepend (file_list, escaped);
-				g_free (uri);
-			} else {
-				file_list = g_list_prepend (file_list, uri);
-			}
- 
- 			if (g_file_test (path, G_FILE_TEST_IS_DIR))
- 				force_user_to_compress = TRUE;
- 
- 			g_free (path);
- 		}
+		uri = g_filename_to_uri (filename, NULL, NULL);
+		g_free (filename);
+		escaped = escape_ampersands (uri);
+
+		if (escaped == NULL) {
+			file_list = g_list_prepend (file_list, uri);
+		} else {
+			file_list = g_list_prepend (file_list, escaped);
+			g_free (uri);
+		}
 	}
 
 	if (file_list == NULL) {
